# coding=utf-8
from unittest import TestCase

from earthdiagnostics.diagnostic import DiagnosticVariableListOption, DiagnosticOptionError
from earthdiagnostics.box import Box
from earthdiagnostics.general.select_levels import SelectLevels
from earthdiagnostics.frequency import Frequencies
from mock import Mock, patch

from earthdiagnostics.modelingrealm import ModelingRealms


class TestSelectLevels(TestCase):

    def setUp(self):
        self.data_manager = Mock()

        self.diags = Mock()
        self.diags.config.experiment.get_chunk_list.return_value = (('20010101', 0, 0), ('20010101', 0, 1))
        self.diags.config.experiment.startdates = ['20010101', ]
        self.diags.config.frequency = Frequencies.monthly

        self.box = Box()
        self.box.min_depth = 0
        self.box.max_depth = 100

    def fake_parse(self, value):
        return value.split('-')

    @patch.object(DiagnosticVariableListOption, 'parse', fake_parse)
    def test_generate_jobs(self):
        jobs = SelectLevels.generate_jobs(self.diags, ['diagnostic', 'atmos', 'var', '0', '20'])
        self.assertEqual(len(jobs), 2)
        self.assertEqual(jobs[0], SelectLevels(self.data_manager, '20010101', 0, 0, ModelingRealms.atmos, 'var',
                                               '', 0, 20))
        self.assertEqual(jobs[1], SelectLevels(self.data_manager, '20010101', 0, 1, ModelingRealms.atmos, 'var',
                                               '', 0, 20))

        jobs = SelectLevels.generate_jobs(self.diags, ['diagnostic', 'atmos', 'var1-var2', '0', '20'])
        self.assertEqual(len(jobs), 4)
        self.assertEqual(jobs[0], SelectLevels(self.data_manager, '20010101', 0, 0, ModelingRealms.atmos, 'var1',
                                               '', 0, 20))
        self.assertEqual(jobs[1], SelectLevels(self.data_manager, '20010101', 0, 1, ModelingRealms.atmos, 'var1',
                                               '', 0, 20))
        self.assertEqual(jobs[2], SelectLevels(self.data_manager, '20010101', 0, 0, ModelingRealms.atmos, 'var2',
                                               '', 0, 20))
        self.assertEqual(jobs[3], SelectLevels(self.data_manager, '20010101', 0, 1, ModelingRealms.atmos, 'var2',
                                               '', 0, 20))

        jobs = SelectLevels.generate_jobs(self.diags, ['diagnostic', 'atmos', 'var', '0', '20', 'grid'])
        self.assertEqual(len(jobs), 2)
        self.assertEqual(jobs[0], SelectLevels(self.data_manager, '20010101', 0, 0, ModelingRealms.atmos, 'var',
                                               'grid', 0, 20))
        self.assertEqual(jobs[1], SelectLevels(self.data_manager, '20010101', 0, 1, ModelingRealms.atmos, 'var',
                                               'grid', 0, 20))

        with self.assertRaises(DiagnosticOptionError):
            SelectLevels.generate_jobs(self.diags, ['diagnostic'])

            with self.assertRaises(DiagnosticOptionError):
                SelectLevels.generate_jobs(self.diags, ['diagnostic', 'atmos', 'var', '0', '20', 'grid', 'extra'])

    def test_str(self):
        mixed = SelectLevels(self.data_manager, '20010101', 0, 0, ModelingRealms.atmos, 'var', 'grid', 0, 20)
        self.assertEqual(str(mixed),
                         'Select levels Startdate: 20010101 Member: 0 Chunk: 0 Variable: atmos:var '
                         'Levels: 0-20 Grid: grid')
