# coding=utf-8
from unittest import TestCase

from earthdiagnostics.diagnostic import DiagnosticVariableListOption, DiagnosticOptionError
from earthdiagnostics.box import Box
from earthdiagnostics.general.simplify_dimensions import SimplifyDimensions
from earthdiagnostics.frequency import Frequencies
from mock import Mock, patch

from earthdiagnostics.modelingrealm import ModelingRealms


class TestSimplifyDimensions(TestCase):

    def setUp(self):
        self.data_manager = Mock()

        self.diags = Mock()
        self.diags.config.experiment.get_chunk_list.return_value = (('20010101', 0, 0), ('20010101', 0, 1))
        self.diags.config.experiment.startdates = ['20010101', ]
        self.diags.config.frequency = Frequencies.monthly
        self.diags.config.data_convention = 'convention'

        self.box = Box()
        self.box.min_depth = 0
        self.box.max_depth = 100

    def fake_parse(self, value):
        return value.split('-')

    @patch.object(DiagnosticVariableListOption, 'parse', fake_parse)
    def test_generate_jobs(self):
        jobs = SimplifyDimensions.generate_jobs(self.diags, ['diagnostic', 'atmos', 'var'])
        self.assertEqual(len(jobs), 2)
        self.assertEqual(jobs[0], SimplifyDimensions(self.data_manager, '20010101', 0, 0,
                                                     ModelingRealms.atmos, 'var', '', 'convention'))
        self.assertEqual(jobs[1], SimplifyDimensions(self.data_manager, '20010101', 0, 1,
                                                     ModelingRealms.atmos, 'var', '', 'convention'))

        jobs = SimplifyDimensions.generate_jobs(self.diags, ['diagnostic', 'atmos', 'var', 'grid'])
        self.assertEqual(len(jobs), 2)
        self.assertEqual(jobs[0], SimplifyDimensions(self.data_manager, '20010101', 0, 0,
                                                     ModelingRealms.atmos, 'var', 'grid', 'convention'))
        self.assertEqual(jobs[1], SimplifyDimensions(self.data_manager, '20010101', 0, 1,
                                                     ModelingRealms.atmos, 'var', 'grid', 'convention'))

        with self.assertRaises(DiagnosticOptionError):
            SimplifyDimensions.generate_jobs(self.diags, ['diagnostic'])

        with self.assertRaises(DiagnosticOptionError):
            SimplifyDimensions.generate_jobs(self.diags, ['diagnostic', 'atmos', 'var', 'grid', 'extra'])

    def test_str(self):
        mixed = SimplifyDimensions(self.data_manager, '20010101', 0, 0, ModelingRealms.atmos, 'var',
                                   'grid', 'convention')
        self.assertEqual(str(mixed),
                         'Simplify dimension Startdate: 20010101 Member: 0 Chunk: 0 Variable: atmos:var '
                         'Grid: grid')
