# coding=utf-8
from unittest import TestCase

from earthdiagnostics.diagnostic import DiagnosticVariableListOption, DiagnosticOptionError
from earthdiagnostics.box import Box
from earthdiagnostics.ocean.mask_land import MaskLand
from mock import Mock, patch

from earthdiagnostics.modelingrealm import ModelingRealms


class TestMaskLand(TestCase):

    def setUp(self):
        self.data_manager = Mock()
        self.diags = Mock()

        self.box = Box()
        self.box.min_lat = 0
        self.box.max_lat = 0
        self.box.min_lon = 0
        self.box.max_lon = 0

        self.diags.config.experiment.get_chunk_list.return_value = (('20010101', 0, 0), ('20010101', 0, 1))

    def fake_parse(self, value):
        if not value:
            raise DiagnosticOptionError
        return value.split('-')

    @patch.object(DiagnosticVariableListOption, 'parse', fake_parse)
    @patch('earthdiagnostics.ocean.mask_land.MaskLand._get_mask')
    def test_generate_jobs(self, get_mask_mock):
        get_mask_mock.return_value = None
        jobs = MaskLand.generate_jobs(self.diags, ['diagnostic', 'ocean', 'var'])
        self.assertEqual(len(jobs), 2)
        self.assertEqual(jobs[0], MaskLand(self.data_manager, '20010101', 0, 0, ModelingRealms.ocean, 'var', 't', ''))
        self.assertEqual(jobs[1], MaskLand(self.data_manager, '20010101', 0, 1, ModelingRealms.ocean, 'var', 't', ''))

        for mask in ('t', 'u', 'v', 'f', 'w'):
            jobs = MaskLand.generate_jobs(self.diags, ['diagnostic', 'ocean', 'var', mask])
            self.assertEqual(len(jobs), 2)
            self.assertEqual(jobs[0], MaskLand(self.data_manager, '20010101', 0, 0, ModelingRealms.ocean, 'var',
                                               mask, ''))
            self.assertEqual(jobs[1], MaskLand(self.data_manager, '20010101', 0, 1, ModelingRealms.ocean, 'var',
                                               mask, ''))

        with self.assertRaises(DiagnosticOptionError):
            MaskLand.generate_jobs(self.diags, ['diagnostic', 'ocean', 'var', 'BAD'])

        jobs = MaskLand.generate_jobs(self.diags, ['diagnostic', 'ocean', 'var', 't', 'grid'])
        self.assertEqual(len(jobs), 2)
        self.assertEqual(jobs[0], MaskLand(self.data_manager, '20010101', 0, 0, ModelingRealms.ocean, 'var',
                                           't', 'grid'))
        self.assertEqual(jobs[1], MaskLand(self.data_manager, '20010101', 0, 1, ModelingRealms.ocean, 'var',
                                           't', 'grid'))

        with self.assertRaises(DiagnosticOptionError):
            MaskLand.generate_jobs(self.diags, ['diagnostic'])
        with self.assertRaises(DiagnosticOptionError):
            MaskLand.generate_jobs(self.diags, ['diagnostic', 'ocean', 'var', 't', 'grid', 'extra'])

    def test_str(self):
        diag = MaskLand(self.data_manager, '20010101', 0, 0, ModelingRealms.ocean, 'var', 't', 'grid')
        self.assertEqual(str(diag), 'Land mask Startdate: 20010101 Member: 0 Chunk: 0 Variable: ocean:var Grid: grid')
