# coding=utf-8
from unittest import TestCase
from earthdiagnostics.ocean.verticalgradient import VerticalGradient
from earthdiagnostics.box import Box
from earthdiagnostics.diagnostic import DiagnosticOptionError, DiagnosticVariableOption
from mock import Mock, patch


class TestVerticalGradient(TestCase):

    def setUp(self):
        self.data_manager = Mock()
        self.diags = Mock()
        self.diags.config.experiment.get_chunk_list.return_value = (('20010101', 0, 0), ('20010101', 0, 1))

    def fake_parse(self, value):
        if not value:
            raise DiagnosticOptionError
        return value

    @patch.object(DiagnosticVariableOption, 'parse', fake_parse)
    def test_generate_jobs(self):

        box = Box()
        box.min_depth = 1
        box.max_depth = 2

        jobs = VerticalGradient.generate_jobs(self.diags, ['diagnostic', 'var'])
        self.assertEqual(len(jobs), 2)
        self.assertEqual(jobs[0], VerticalGradient(self.data_manager, '20010101', 0, 0, 'var', box))
        self.assertEqual(jobs[1], VerticalGradient(self.data_manager, '20010101', 0, 1, 'var', box))

        box = Box()
        box.min_depth = 2
        box.max_depth = 2

        jobs = VerticalGradient.generate_jobs(self.diags, ['diagnostic', 'var', '2'])
        self.assertEqual(len(jobs), 2)
        self.assertEqual(jobs[0], VerticalGradient(self.data_manager, '20010101', 0, 0, 'var', box))
        self.assertEqual(jobs[1], VerticalGradient(self.data_manager, '20010101', 0, 1, 'var', box))

        box = Box()
        box.min_depth = 1
        box.max_depth = 10

        jobs = VerticalGradient.generate_jobs(self.diags, ['diagnostic', 'var', '1', 10])
        self.assertEqual(len(jobs), 2)
        self.assertEqual(jobs[0], VerticalGradient(self.data_manager, '20010101', 0, 0, 'var', box))
        self.assertEqual(jobs[1], VerticalGradient(self.data_manager, '20010101', 0, 1, 'var', box))

        with self.assertRaises(DiagnosticOptionError):
            VerticalGradient.generate_jobs(self.diags, ['diagnostic'])

        with self.assertRaises(DiagnosticOptionError):
            VerticalGradient.generate_jobs(self.diags, ['diagnostic', 'var', '1', '10', 'extra'])

    def test_str(self):
        box = Box()
        box.min_depth = 1
        box.max_depth = 10

        diag = VerticalGradient(self.data_manager, '20010101', 0, 0, 'var', box)
        self.assertEqual(str(diag), 'Vertical gradient Startdate: 20010101 Member: 0 Chunk: 0 Variable: var Box: 1-10')
