# coding=utf-8
from unittest import TestCase

from earthdiagnostics.statistics.climatologicalpercentile import ClimatologicalPercentile
from earthdiagnostics.diagnostic import DiagnosticVariableOption
from mock import Mock, patch

from earthdiagnostics.modelingrealm import ModelingRealms


class TestClimatologicalPercentile(TestCase):

    def setUp(self):
        self.data_manager = Mock()
        self.data_manager.variable_list.get_variable.return_value = None

        self.diags = Mock()
        self.diags.data_manager = self.data_manager

    def fake_parse(self, value):
        return value

    @patch.object(DiagnosticVariableOption, 'parse', fake_parse)
    def test_generate_jobs(self):
        jobs = ClimatologicalPercentile.generate_jobs(self.diags, ['climpercent', 'ocean', 'var', '2000', '2001', '11'])
        self.assertEqual(len(jobs), 1)
        self.assertEqual(jobs[0], ClimatologicalPercentile(self.data_manager, ModelingRealms.ocean, 'var',
                                                           2000, 2001, 11,
                                                           self.diags.config.experiment))

        with self.assertRaises(Exception):
            ClimatologicalPercentile.generate_jobs(self.diags, ['climpercent'])
        with self.assertRaises(Exception):
            ClimatologicalPercentile.generate_jobs(self.diags, ['climpercent', 'ocean', 'var', '2000', '2001', '11',
                                                                'extra'])

    def test_str(self):
        diagnostic = ClimatologicalPercentile(self.data_manager, ModelingRealms.ocean, 'var',
                                              2000, 2001, 11, self.diags.config.experiment)

        self.assertEqual(str(diagnostic),
                         'Climatological percentile Variable: ocean:var Period: 2000-2001 Forecast month: 11')
