# coding=utf-8
from unittest import TestCase

from earthdiagnostics.statistics.discretize import Discretize
from earthdiagnostics.diagnostic import DiagnosticVariableOption
from mock import Mock, patch

from earthdiagnostics.modelingrealm import ModelingRealms


class TestClimatologicalPercentile(TestCase):

    def setUp(self):
        self.data_manager = Mock()
        self.data_manager.variable_list.get_variable.return_value = None

        self.diags = Mock()
        self.diags.data_manager = self.data_manager
        self.diags.config.experiment.startdates = ('20000101', '20010101')

    def fake_parse(self, value):
        return value

    @patch.object(DiagnosticVariableOption, 'parse', fake_parse)
    def test_generate_jobs(self):
        jobs = Discretize.generate_jobs(self.diags, ['discretize', 'ocean', 'var'])
        self.assertEqual(len(jobs), 2)
        self.assertEqual(jobs[0], Discretize(self.data_manager, '20000101', ModelingRealms.ocean, 'var',
                                             2000, float('nan'), float('nan')))
        self.assertEqual(jobs[1], Discretize(self.data_manager, '20010101', ModelingRealms.ocean, 'var',
                                             2000, float('nan'), float('nan')))

        jobs = Discretize.generate_jobs(self.diags, ['discretize', 'ocean', 'var', '500'])
        self.assertEqual(jobs[0], Discretize(self.data_manager, '20000101', ModelingRealms.ocean, 'var',
                                             500, float('nan'), float('nan')))
        self.assertEqual(jobs[1], Discretize(self.data_manager, '20010101', ModelingRealms.ocean, 'var',
                                             500, float('nan'), float('nan')))

        jobs = Discretize.generate_jobs(self.diags, ['discretize', 'ocean', 'var', '500', '0', '40'])
        self.assertEqual(jobs[0], Discretize(self.data_manager, '20000101', ModelingRealms.ocean, 'var',
                                             500, 0, 40))
        self.assertEqual(jobs[1], Discretize(self.data_manager, '20010101', ModelingRealms.ocean, 'var',
                                             500, 0, 40))

        with self.assertRaises(Exception):
            Discretize.generate_jobs(self.diags, ['discretize'])
        with self.assertRaises(Exception):
            Discretize.generate_jobs(self.diags, ['discretize', 'ocean', 'var', '500', '0', '40', 'extra'])

    def test_str(self):
        diagnostic = Discretize(self.data_manager, '20000101', ModelingRealms.ocean, 'var', 2000, 10, 40)

        self.assertEqual(str(diagnostic),
                         'Discretizing variable: ocean:var Startdate: 20000101 Bins: 2000 Range: [10, 40]')

        diagnostic = Discretize(self.data_manager, '20000101', ModelingRealms.ocean, 'var', 2000,
                                float('nan'), float('nan'))

        self.assertEqual(str(diagnostic),
                         'Discretizing variable: ocean:var Startdate: 20000101 Bins: 2000 Range: [None, None]')
