# coding=utf-8
import os
from unittest import TestCase

import mock

from earthdiagnostics.cdftools import CDFTools


# noinspection PyUnusedLocal
def bad_file(path, access=None):
    return not os.path.basename(path).startswith('bad')


class TestCDFTools(TestCase):
    # noinspection PyUnusedLocal
    @mock.patch('os.path.isfile', side_effect=bad_file)
    @mock.patch('os.access', side_effect=bad_file)
    @mock.patch('earthdiagnostics.utils.Utils.execute_shell_command')
    def test_run(self, mock_path, mock_exists, execute_mock):
        self.cdftools = CDFTools('')
        execute_mock.return_value = ['Command output']
        with self.assertRaises(ValueError):
            self.cdftools.run('badcommand', input_file='input_file', output_file='output_file')
        with self.assertRaises(ValueError):
            self.cdftools.run('command', input_file='badinput_file', output_file='output_file')
        with self.assertRaises(ValueError):
            self.cdftools.run('command', input_file=['input_file', 'badinput_file'], output_file='output_file')
        with self.assertRaises(ValueError):
            self.cdftools.run('command', input_file='input_file', output_file='input_file')
        with self.assertRaises(Exception):
            self.cdftools.run('command', input_file='input_file', output_file='badoutput_file')

        self.cdftools.run('command', input_file='input_file', output_file='output_file')
        self.cdftools.run('command', input_option='-i', input_file='input_file', output_file='output_file')
        self.cdftools.run('command', input_file='input_file')
        self.cdftools.run('command', input_file=None)
        self.cdftools.run('command', input_file=['input_file', 'input_file2'])
        self.cdftools.run('command', input_file='input_file', options='-o -p')
        self.cdftools.run('command', input_file='input_file', options=('-o', '-p'))

    # noinspection PyUnusedLocal
    @mock.patch('os.path.isfile', side_effect=bad_file)
    @mock.patch('os.access', side_effect=bad_file)
    @mock.patch('earthdiagnostics.utils.Utils.execute_shell_command')
    def test_run_with_path(self, mock_path, mock_exists, execute_mock):
        self.cdftools = CDFTools('/some/path')
        execute_mock.return_value = ['Command output']
        with self.assertRaises(ValueError):
            self.cdftools.run('badcommand', input_file='input_file', output_file='output_file')
        with self.assertRaises(ValueError):
            self.cdftools.run('command', input_file='badinput_file', output_file='output_file')
        with self.assertRaises(ValueError):
            self.cdftools.run('command', input_file=['input_file', 'badinput_file'], output_file='output_file')
        with self.assertRaises(ValueError):
            self.cdftools.run('command', input_file='input_file', output_file='input_file')
        with self.assertRaises(Exception):
            self.cdftools.run('command', input_file='input_file', output_file='badoutput_file')

        self.cdftools.run('command', input_file='input_file', output_file='output_file')
        self.cdftools.run('command', input_option='-i', input_file='input_file', output_file='output_file')
        self.cdftools.run('command', input_file='input_file')
        self.cdftools.run('command', input_file=None)
        self.cdftools.run('command', input_file=['input_file', 'input_file2'])
        self.cdftools.run('command', input_file='input_file', options='-o -p')
        self.cdftools.run('command', input_file='input_file', options=('-o', '-p'))
