# coding=utf-8
from unittest import TestCase
from mock import Mock

from earthdiagnostics.datafile import UnitConversion, DataFile, LocalStatus, StorageStatus


class TestConversion(TestCase):

    def test__init(self):
        conversion = UnitConversion('km', 'm', 1000, 0)
        self.assertEqual(conversion.source, 'km')
        self.assertEqual(conversion.destiny, 'm')
        self.assertEqual(conversion.factor, 1000)
        self.assertEqual(conversion.offset, 0)

    def test_add_conversion(self):
        UnitConversion._dict_conversions = dict()
        conversion = UnitConversion('km', 'm', 1000, 0)
        UnitConversion.add_conversion(conversion)
        self.assertIs(UnitConversion._dict_conversions[('km', 'm')], conversion)
        UnitConversion._dict_conversions = dict()

    def test_get_factor_offset(self):
        UnitConversion._dict_conversions = dict()
        conversion = UnitConversion('km', 'm', 1000, 0)
        UnitConversion.add_conversion(conversion)
        self.assertEqual(UnitConversion.get_conversion_factor_offset('km', 'm'), (1000, 0))
        self.assertEqual(UnitConversion.get_conversion_factor_offset('m', 'km'), (1 / 1000.0, 0))
        self.assertEqual(UnitConversion.get_conversion_factor_offset('1e3 m', 'km'), (1, 0))
        self.assertEqual(UnitConversion.get_conversion_factor_offset('10^3 m', 'km'), (1, 0))
        self.assertEqual(UnitConversion.get_conversion_factor_offset('km', '1e3 m'), (1, 0))
        self.assertEqual(UnitConversion.get_conversion_factor_offset('km', '10^3 m'), (1, 0))
        self.assertEqual(UnitConversion.get_conversion_factor_offset('m', 'm'), (1, 0))
        self.assertEqual(UnitConversion.get_conversion_factor_offset('m²', 'km'), (None, None))

        UnitConversion._dict_conversions = dict()


class TestDatafile(TestCase):

    def setUp(self):
        self.data_file = DataFile()

    def test_download_required(self):
        self.assertFalse(self.data_file.download_required())

        self.data_file.local_status = LocalStatus.READY
        self.assertFalse(self.data_file.download_required())

        self.data_file.local_status = LocalStatus.COMPUTING
        self.assertFalse(self.data_file.download_required())

        self.data_file.local_status = LocalStatus.PENDING
        self.data_file.storage_status = StorageStatus.READY
        self.assertTrue(self.data_file.download_required())

        self.data_file.local_status = LocalStatus.PENDING
        self.data_file.storage_status = StorageStatus.UPLOADING
        self.assertFalse(self.data_file.download_required())

    def test_upload_required(self):
        self.assertFalse(self.data_file.upload_required())

        self.data_file.local_status = LocalStatus.READY
        self.assertFalse(self.data_file.upload_required())

        self.data_file.local_status = LocalStatus.COMPUTING
        self.assertFalse(self.data_file.upload_required())

        self.data_file.local_status = LocalStatus.READY
        self.data_file.storage_status = StorageStatus.PENDING
        self.assertTrue(self.data_file.upload_required())

        self.data_file.local_status = LocalStatus.PENDING
        self.data_file.storage_status = StorageStatus.READY
        self.assertFalse(self.data_file.upload_required())

    def test_add_modifier(self):
        self.assertFalse(self.data_file.has_modifiers())
        self.data_file.add_modifier(Mock())
        self.assertTrue(self.data_file.has_modifiers())
